@@ -16,26 +16,8 @@
     #define CX_PATH_FORMAT              L"%ls/lib/library.zip:%ls/lib"
 #endif
 
-// define code for Python 2.x to set sys.path
-// can't use os.path, due to only `sys` module loaded at this point
-#define CX_SET_PATH_CODE \
-    "import sys\n" \
-    "lib = sys.exec_prefix + '/lib'\n" \
-    "sys.path = [lib, lib + '/library.zip']\n"
-
-// define names that will work for both Python 2 and 3
-#if PY_MAJOR_VERSION >= 3
-    #define cxString_FromString         PyUnicode_FromString
-    #define cxWin_GetModuleFileName     GetModuleFileNameW
-    #define cxWin_PathRemoveFileSpec    PathRemoveFileSpecW
-#else
-    #define cxString_FromString         PyString_FromString
-    #define cxWin_GetModuleFileName     GetModuleFileNameA
-    #define cxWin_PathRemoveFileSpec    PathRemoveFileSpecA
-#endif
-
 // global variables (used for simplicity)
-#if defined(MS_WINDOWS) && PY_MAJOR_VERSION >= 3
+#if defined(MS_WINDOWS)
     static wchar_t g_ExecutableName[MAXPATHLEN + 1];
     static wchar_t g_ExecutableDirName[MAXPATHLEN + 1];
 #else
@@ -56,11 +38,11 @@ static int SetExecutableName(
     const char *argv0)                  // script to execute
 {
 #ifdef MS_WINDOWS
-    if (!cxWin_GetModuleFileName(NULL, g_ExecutableName, MAXPATHLEN + 1))
+    if (!GetModuleFileNameW(NULL, g_ExecutableName, MAXPATHLEN + 1))
         return FatalError("Unable to get executable name!");
     memcpy(g_ExecutableDirName, g_ExecutableName,
-            (MAXPATHLEN + 1) * (PY_MAJOR_VERSION == 2 ? sizeof(char) : sizeof(wchar_t)));
-    cxWin_PathRemoveFileSpec(g_ExecutableDirName);
+            (MAXPATHLEN + 1) * sizeof(wchar_t));
+    PathRemoveFileSpecW(g_ExecutableDirName);
 
 #else
 
@@ -82,11 +64,9 @@ static int SetExecutableName(
         argv0Size = strlen(argv0);
         while (1) {
             tempPtr = strchr(ptr, DELIM);
-            if (tempPtr)
-                size = tempPtr - ptr;
-            else size = strlen(ptr);
+            size = (tempPtr) ? (size_t) (tempPtr - ptr) : strlen(ptr);
             if (size + argv0Size + 1 <= MAXPATHLEN) {
-                strncpy(executableName, ptr, size);
+                strncpy(executableName, ptr, PATH_MAX + 1);
                 executableName[size] = SEP;
                 executableName[size + 1] = '\0';
                 strcat(executableName, argv0);
@@ -121,8 +101,6 @@ static int SetExecutableName(
 }
 
 
-#if PY_MAJOR_VERSION >= 3
-
 #ifdef MS_WINDOWS
 //-----------------------------------------------------------------------------
 // InitializePython()
@@ -215,32 +193,6 @@ static int InitializePython(int argc, char **argv)
 
 #endif
 
-#else
-
-//-----------------------------------------------------------------------------
-// InitializePython()
-//   Initialize Python for Python 2.x.
-//-----------------------------------------------------------------------------
-static int InitializePython(int argc, char **argv)
-{
-    // determine executable name
-    if (SetExecutableName(argv[0]) < 0)
-        return -1;
-
-    // initialize Python
-    Py_NoSiteFlag = 1;
-    Py_FrozenFlag = 1;
-    Py_IgnoreEnvironmentFlag = 1;
-    Py_SetPythonHome(g_ExecutableDirName);
-    Py_SetProgramName(g_ExecutableName);
-    Py_Initialize();
-    PySys_SetArgv(argc, argv);
-
-    return 0;
-}
-
-#endif
-
 
 //-----------------------------------------------------------------------------
 // ExecuteScript()
@@ -250,12 +202,7 @@ static int ExecuteScript(void)
 {
     PyObject *name, *module, *function, *result;
 
-#if PY_MAJOR_VERSION < 3
-    // only for Python 2.7 since path already set using C API for Python 3
-    PyRun_SimpleString(CX_SET_PATH_CODE);
-#endif
-
-    name = cxString_FromString("__startup__");
+    name = PyUnicode_FromString("__startup__");
     if (!name)
         return FatalError("Cannot create string for startup module name!");
 
