@@ -4,22 +4,39 @@
 //-----------------------------------------------------------------------------
 
 #include <Python.h>
+#include <locale.h>
 #include <windows.h>
 #include <Winsvc.h>
+#include <shlwapi.h>
 #include <cx_Logging.h>
 
+// define macro for convenience
+#define cxString_FromAscii(str) \
+    PyUnicode_DecodeASCII(str, strlen(str), NULL)
+
 // define constants
-#define LOGGING_SECTION_NAME            "Logging"
-#define LOGGING_FILE_NAME_KEY           "FileName"
-#define LOGGING_LEVEL_KEY               "Level"
-#define LOGGING_MAX_FILES_KEY           "MaxFiles"
-#define LOGGING_MAX_FILE_SIZE_KEY       "MaxFileSize"
-#define LOGGING_PREFIX_KEY              "Prefix"
-#define SERVICE_MODULE_NAME             "MODULE_NAME"
-#define SERVICE_CLASS_NAME              "CLASS_NAME"
-#define SERVICE_NAME                    "NAME"
-#define SERVICE_DISPLAY_NAME            "DISPLAY_NAME"
-#define SERVICE_DESCRIPTION             "DESCRIPTION"
+#define CX_LOGGING_SECTION_NAME         L"Logging"
+#define CX_LOGGING_FILE_NAME_KEY        L"FileName"
+#define CX_LOGGING_LEVEL_KEY            L"Level"
+#define CX_LOGGING_MAX_FILES_KEY        L"MaxFiles"
+#define CX_LOGGING_MAX_FILE_SIZE_KEY    L"MaxFileSize"
+#define CX_LOGGING_PREFIX_KEY           L"Prefix"
+#define CX_LOGGING_PREFIX_SIZE          100
+#define CX_LOGGING_PREFIX_DEFAULT       L"[%i] %d %t"
+#define CX_SERVICE_MODULE_NAME          "MODULE_NAME"
+#define CX_SERVICE_CLASS_NAME           "CLASS_NAME"
+#define CX_SERVICE_NAME                 "NAME"
+#define CX_SERVICE_DISPLAY_NAME         "DISPLAY_NAME"
+#define CX_SERVICE_DESCRIPTION          "DESCRIPTION"
+#define CX_SERVICE_AUTO_START           "AUTO_START"
+#define CX_SERVICE_SESSION_CHANGES      "SESSION_CHANGES"
+#define CX_SERVICE_LOGGING_EXTENSION    L".log"
+#define CX_SERVICE_INI_EXTENSION        L".ini"
+#define CX_SERVICE_INSTALL_OPTION       L"--install"
+#define CX_SERVICE_UNINSTALL_OPTION     L"--uninstall"
+#define CX_SERVICE_INSTALL_USAGE        "%ls --install <NAME> [<CONFIGFILE>]"
+#define CX_SERVICE_UNINSTALL_USAGE      "%ls --uninstall <NAME>"
+#define CX_SERVICE_INIT_MESSAGE         "initializing with config file %ls"
 
 // the following was copied from cx_Interface.c, which is where this
 // declaration normally happens
@@ -27,12 +44,15 @@
     #define PATH_MAX _MAX_PATH
 #endif
 
+
 //define structure for holding information about the service
 typedef struct {
     PyObject *cls;
     PyObject *nameFormat;
     PyObject *displayNameFormat;
     PyObject *description;
+    DWORD startType;
+    int sessionChanges;
 } udt_ServiceInfo;
 
 // define globals
@@ -40,14 +60,14 @@ static HANDLE gControlEvent = NULL;
 static SERVICE_STATUS_HANDLE gServiceHandle;
 static PyInterpreterState *gInterpreterState = NULL;
 static PyObject *gInstance = NULL;
-static char gIniFileName[PATH_MAX + 1];
+static wchar_t gIniFileName[PATH_MAX + 1];
 
 //-----------------------------------------------------------------------------
 // FatalError()
 //   Called when an attempt to initialize the module zip fails.
 //-----------------------------------------------------------------------------
 static int FatalError(
-    const char *message)		// message to print
+    const char *message)		        // message to print
 {
     return LogPythonException(message);
 }
@@ -63,20 +83,22 @@ static int FatalScriptError(void)
 }
 
 #include "Common.c"
-#include "BaseModules.c"
 
 //-----------------------------------------------------------------------------
 // Service_SetStatus()
 //   Set the status for the service.
 //-----------------------------------------------------------------------------
 static int Service_SetStatus(
-    DWORD status)			// status to set
+    udt_ServiceInfo* info,              // service information
+    DWORD status)			            // status to set
 {
     SERVICE_STATUS serviceStatus;
 
     serviceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
     serviceStatus.dwCurrentState = status;
     serviceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;
+    if (info->sessionChanges)
+        serviceStatus.dwControlsAccepted |= SERVICE_ACCEPT_SESSIONCHANGE;
     serviceStatus.dwWin32ExitCode = 0;
     serviceStatus.dwServiceSpecificExitCode = 0;
     serviceStatus.dwCheckPoint = 0;
@@ -94,13 +116,14 @@ static int Service_SetStatus(
 // the main thread is ended or the control GUI does not understand that the
 // service has ended.
 //-----------------------------------------------------------------------------
-static int Service_Stop()
+static int Service_Stop(
+    udt_ServiceInfo* info)              // service information
 {
     PyThreadState *threadState;
     PyObject *result;
 
     // indicate that the service is being stopped
-    if (Service_SetStatus(SERVICE_STOP_PENDING) < 0)
+    if (Service_SetStatus(info, SERVICE_STOP_PENDING) < 0)
         return LogWin32Error(GetLastError(), "cannot set service as stopping");
 
     // create event for the main thread to wait on for the control thread
@@ -126,7 +149,7 @@ static int Service_Stop()
     PyThreadState_Delete(threadState);
 
     // indicate that the service has stopped
-    if (Service_SetStatus(SERVICE_STOPPED) < 0)
+    if (Service_SetStatus(info, SERVICE_STOPPED) < 0)
         return LogWin32Error(GetLastError(), "cannot set service as stopped");
 
     // now set the control event
@@ -137,17 +160,65 @@ static int Service_Stop()
 }
 
 
+//-----------------------------------------------------------------------------
+// Service_SessionChange()
+//   Called when a session has changed.
+//-----------------------------------------------------------------------------
+static int Service_SessionChange(
+    DWORD sessionId,                    // session that has changed
+    DWORD eventType)                    // event type
+{
+    PyThreadState *threadState;
+    PyObject *result;
+
+    // create a new Python thread and acquire the global interpreter lock
+    threadState = PyThreadState_New(gInterpreterState);
+    if (!threadState)
+        return LogPythonException("unable to create new thread state");
+    PyEval_AcquireThread(threadState);
+
+    // call Python method
+    result = PyObject_CallMethod(gInstance, "SessionChanged", "ii",
+            sessionId, eventType);
+    if (!result)
+        return LogPythonException("exception calling SessionChanged method");
+    Py_DECREF(result);
+
+    // destroy the Python thread and release the global interpreter lock
+    PyThreadState_Clear(threadState);
+    PyEval_ReleaseThread(threadState);
+    PyThreadState_Delete(threadState);
+
+    return 0;
+}
+
+
 //-----------------------------------------------------------------------------
 // Service_Control()
 //   Function for controlling a service. Note that the controlling thread
 // must be ended before the main thread is ended or the control GUI does not
 // understand that the service has ended.
 //-----------------------------------------------------------------------------
-static void WINAPI Service_Control(
-    DWORD controlCode)			// control code
+static DWORD WINAPI Service_Control(
+    DWORD controlCode,			        // control code
+    DWORD eventType,                    // event type
+    LPVOID eventData,                   // event data
+    LPVOID context)                     // context
 {
-    if (controlCode == SERVICE_CONTROL_STOP)
-        Service_Stop();
+    udt_ServiceInfo *serviceInfo = (udt_ServiceInfo*) context;
+    WTSSESSION_NOTIFICATION *sessionInfo;
+
+    switch (controlCode) {
+        case SERVICE_CONTROL_STOP:
+            Service_Stop(serviceInfo);
+            break;
+        case SERVICE_CONTROL_SESSIONCHANGE:
+            sessionInfo = (WTSSESSION_NOTIFICATION*) eventData;
+            Service_SessionChange(sessionInfo->dwSessionId, eventType);
+            break;
+    }
+
+    return NO_ERROR;
 }
 
 
@@ -155,40 +226,41 @@ static void WINAPI Service_Control(
 // Service_StartLogging()
 //   Initialize logging for the service.
 //-----------------------------------------------------------------------------
-static int Service_StartLogging(
-    const char *fileName)		// name of file for defaults
+static int Service_StartLogging(void)
 {
-    char defaultLogFileName[PATH_MAX + 1], logFileName[PATH_MAX + 1];
-    unsigned size, logLevel, maxFiles, maxFileSize;
-    char *ptr, prefix[100];
+    wchar_t defaultLogFileName[PATH_MAX + 1], logFileName[PATH_MAX + 1];
+    unsigned logLevel, maxFiles, maxFileSize;
+    wchar_t *ptr, prefix[CX_LOGGING_PREFIX_SIZE];
+    size_t size;
 
     // determine the default log file name and ini file name
-    ptr = strrchr(fileName, '.');
+    ptr = wcsrchr(g_ExecutableName, '.');
     if (ptr)
-       size = ptr - fileName;
-    else size = strlen(fileName);
-    strcpy(defaultLogFileName, fileName);
-    strcpy(&defaultLogFileName[size], ".log");
-    if (strlen(gIniFileName) == 0) {
-        strcpy(gIniFileName, fileName);
-        strcpy(&gIniFileName[size], ".ini");
+       size = ptr - g_ExecutableName;
+    else size = wcslen(g_ExecutableName);
+    wcscpy(defaultLogFileName, g_ExecutableName);
+    wcscpy(&defaultLogFileName[size], CX_SERVICE_LOGGING_EXTENSION);
+    if (wcslen(gIniFileName) == 0) {
+        wcscpy(gIniFileName, g_ExecutableName);
+        wcscpy(&gIniFileName[size], CX_SERVICE_INI_EXTENSION);
     }
 
     // read the entries from the ini file
-    logLevel = GetPrivateProfileInt(LOGGING_SECTION_NAME, LOGGING_LEVEL_KEY,
-            LOG_LEVEL_ERROR, gIniFileName);
-    GetPrivateProfileString(LOGGING_SECTION_NAME, LOGGING_FILE_NAME_KEY,
-            defaultLogFileName, logFileName, sizeof(logFileName),
-            gIniFileName);
-    maxFiles = GetPrivateProfileInt(LOGGING_SECTION_NAME,
-            LOGGING_MAX_FILES_KEY, 1, gIniFileName);
-    maxFileSize = GetPrivateProfileInt(LOGGING_SECTION_NAME,
-            LOGGING_MAX_FILE_SIZE_KEY, DEFAULT_MAX_FILE_SIZE, gIniFileName);
-    GetPrivateProfileString(LOGGING_SECTION_NAME, LOGGING_PREFIX_KEY,
-            "[%i] %d %t", prefix, sizeof(prefix), gIniFileName);
+    logLevel = GetPrivateProfileIntW(CX_LOGGING_SECTION_NAME,
+            CX_LOGGING_LEVEL_KEY, LOG_LEVEL_ERROR, gIniFileName);
+    GetPrivateProfileStringW(CX_LOGGING_SECTION_NAME,
+            CX_LOGGING_FILE_NAME_KEY, defaultLogFileName, logFileName,
+            sizeof(logFileName), gIniFileName);
+    maxFiles = GetPrivateProfileIntW(CX_LOGGING_SECTION_NAME,
+            CX_LOGGING_MAX_FILES_KEY, 1, gIniFileName);
+    maxFileSize = GetPrivateProfileIntW(CX_LOGGING_SECTION_NAME,
+            CX_LOGGING_MAX_FILE_SIZE_KEY, DEFAULT_MAX_FILE_SIZE, gIniFileName);
+    GetPrivateProfileStringW(CX_LOGGING_SECTION_NAME,
+            CX_LOGGING_PREFIX_KEY, CX_LOGGING_PREFIX_DEFAULT, prefix,
+            CX_LOGGING_PREFIX_SIZE, gIniFileName);
 
     // start the logging process
-    return StartLogging(logFileName, logLevel, maxFiles, maxFileSize, prefix);
+    return StartLoggingW(logFileName, logLevel, maxFiles, maxFileSize, prefix);
 }
 
 
@@ -197,25 +269,13 @@ static int Service_StartLogging(
 //   Setup Python usage for the service.
 //-----------------------------------------------------------------------------
 static int Service_SetupPython(
-    char *programName,                  // name of the program (argv[0])
     udt_ServiceInfo *info)              // info about service (OUT)
 {
     PyObject *module, *serviceModule, *temp;
     PyThreadState *threadState;
-    char *fileName;
-
-    // initialize Python
-    Py_NoSiteFlag = 1;
-    Py_FrozenFlag = 1;
-    Py_IgnoreEnvironmentFlag = 1;
-    PyImport_FrozenModules = gFrozenModules;
-    Py_SetPythonHome("");
-    Py_SetProgramName(programName);
-    fileName = Py_GetProgramFullPath();
-    Py_Initialize();
 
     // initialize logging
-    if (Service_StartLogging(fileName) < 0)
+    if (Service_StartLogging() < 0)
         return -1;
 
     // ensure threading is initialized and interpreter state saved
@@ -223,7 +283,7 @@ static int Service_SetupPython(
     threadState = PyThreadState_Swap(NULL);
     if (!threadState) {
         LogMessage(LOG_LEVEL_ERROR, "cannot set up interpreter state");
-        Service_SetStatus(SERVICE_STOPPED);
+        Service_SetStatus(info, SERVICE_STOPPED);
         return -1;
     }
     gInterpreterState = threadState->interp;
@@ -231,7 +291,7 @@ static int Service_SetupPython(
 
     // running base script
     LogMessage(LOG_LEVEL_DEBUG, "running base Python script");
-    if (ExecuteScript(fileName) < 0)
+    if (ExecuteScript() < 0)
         return -1;
 
     // acquire the __main__ module
@@ -240,24 +300,39 @@ static int Service_SetupPython(
         return LogPythonException("unable to import __main__");
 
     // determine name to use for the service
-    info->nameFormat = PyObject_GetAttrString(module, SERVICE_NAME);
+    info->nameFormat = PyObject_GetAttrString(module, CX_SERVICE_NAME);
     if (!info->nameFormat)
         return LogPythonException("cannot locate service name");
 
     // determine display name to use for the service
     info->displayNameFormat = PyObject_GetAttrString(module,
-            SERVICE_DISPLAY_NAME);
+            CX_SERVICE_DISPLAY_NAME);
     if (!info->displayNameFormat)
         return LogPythonException("cannot locate service display name");
 
     // determine description to use for the service (optional)
-    info->description = PyObject_GetAttrString(module,
-            SERVICE_DESCRIPTION);
+    info->description = PyObject_GetAttrString(module, CX_SERVICE_DESCRIPTION);
     if (!info->description)
         PyErr_Clear();
 
+    // determine if service should be automatically started (optional)
+    info->startType = SERVICE_DEMAND_START;
+    temp = PyObject_GetAttrString(module, CX_SERVICE_AUTO_START);
+    if (!temp)
+        PyErr_Clear();
+    else if (temp == Py_True)
+        info->startType = SERVICE_AUTO_START;
+
+    // determine if service should monitor session changes (optional)
+    info->sessionChanges = 0;
+    temp = PyObject_GetAttrString(module, CX_SERVICE_SESSION_CHANGES);
+    if (!temp)
+        PyErr_Clear();
+    else if (temp == Py_True)
+        info->sessionChanges = 1;
+
     // import the module which implements the service
-    temp = PyObject_GetAttrString(module, SERVICE_MODULE_NAME);
+    temp = PyObject_GetAttrString(module, CX_SERVICE_MODULE_NAME);
     if (!temp)
         return LogPythonException("cannot locate service module name");
     serviceModule = PyImport_Import(temp);
@@ -266,7 +341,7 @@ static int Service_SetupPython(
         return LogPythonException("cannot import service module");
 
     // create an instance of the class which implements the service
-    temp = PyObject_GetAttrString(module, SERVICE_CLASS_NAME);
+    temp = PyObject_GetAttrString(module, CX_SERVICE_CLASS_NAME);
     if (!temp)
         return LogPythonException("cannot locate service class name");
     info->cls = PyObject_GetAttr(serviceModule, temp);
@@ -283,47 +358,70 @@ static int Service_SetupPython(
 //   Install the service with the given name.
 //-----------------------------------------------------------------------------
 static int Service_Install(
-    char *programName,                  // name of program being run
-    char *name,                         // name of service
-    char *configFileName)               // name of configuration file or NULL
+    wchar_t *name,                      // name of service
+    wchar_t *configFileName)            // name of configuration file or NULL
 {
-    PyObject *fullName, *displayName, *formatArgs, *command, *commandArgs;
-    char fullPathConfigFileName[PATH_MAX + 1];
+    PyObject *executableNameObj, *configFileNameObj, *formatObj, *nameObj;
+    PyObject *fullName, *displayName, *formatArgs, *command;
+    wchar_t fullPathConfigFileName[PATH_MAX + 1];
     SC_HANDLE managerHandle, serviceHandle;
-    SERVICE_DESCRIPTIONA sd;
+    SERVICE_DESCRIPTIONW sd;
     udt_ServiceInfo info;
 
     // set up Python
-    if (Service_SetupPython(programName, &info) < 0)
+    if (Service_SetupPython(&info) < 0)
         return -1;
 
     // determine name and display name to use for the service
-    formatArgs = Py_BuildValue("(s)", name);
+    nameObj = PyUnicode_FromWideChar(name, -1);
+    if (!nameObj)
+        return LogPythonException("cannot create service name obj");
+    formatArgs = PyTuple_Pack(1, nameObj);
     if (!formatArgs)
         return LogPythonException("cannot create service name tuple");
-    fullName = PyString_Format(info.nameFormat, formatArgs);
+    fullName = PyUnicode_Format(info.nameFormat, formatArgs);
     if (!fullName)
         return LogPythonException("cannot create service name");
-    displayName = PyString_Format(info.displayNameFormat, formatArgs);
+    displayName = PyUnicode_Format(info.displayNameFormat, formatArgs);
     if (!displayName)
         return LogPythonException("cannot create display name");
+    Py_CLEAR(formatArgs);
+    Py_CLEAR(nameObj);
 
     // determine command to use for the service
-    command = PyString_FromFormat("\"%s\"", Py_GetProgramFullPath());
-    if (!command)
-        return LogPythonException("cannot create command");
-    if (configFileName) {
-        if (!_fullpath(fullPathConfigFileName, configFileName,
-                sizeof(fullPathConfigFileName)))
+    executableNameObj = PyUnicode_FromWideChar(g_ExecutableName, -1);
+    if (!executableNameObj)
+        return LogPythonException("cannot create executable name obj");
+    if (!configFileName) {
+        formatObj = cxString_FromAscii("\"%s\"");
+        if (!formatObj)
+            return LogPythonException("cannot create format string");
+        formatArgs = PyTuple_Pack(1, executableNameObj);
+        if (!formatArgs)
+            return LogPythonException("cannot create short command tuple");
+    } else {
+        Py_CLEAR(formatArgs);
+        if (!_wfullpath(fullPathConfigFileName, configFileName,
+                PATH_MAX + 1))
             return LogWin32Error(GetLastError(),
                     "cannot calculate absolute path of config file name");
-        commandArgs = PyString_FromFormat(" \"%s\"", fullPathConfigFileName);
-        if (!commandArgs)
-            return LogPythonException("cannot create command args");
-        PyString_Concat(&command, commandArgs);
-        if (!command)
-            return LogPythonException("cannot append args to command");
+        formatObj = cxString_FromAscii("\"%s\" \"%s\"");
+        if (!formatObj)
+            return LogPythonException("cannot create format string");
+        configFileNameObj = PyUnicode_FromWideChar(fullPathConfigFileName, -1);
+        if (!configFileNameObj)
+            return LogPythonException("cannot create config file name string");
+        formatArgs = PyTuple_Pack(2, executableNameObj, configFileNameObj);
+        if (!formatArgs)
+            return LogPythonException("cannot create long command tuple");
+        Py_CLEAR(configFileNameObj);
     }
+    Py_CLEAR(executableNameObj);
+    command = PyUnicode_Format(formatObj, formatArgs);
+    if (!command)
+        return LogPythonException("cannot create command");
+    Py_CLEAR(formatObj);
+    Py_CLEAR(formatArgs);
 
     // open up service control manager
     managerHandle = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
@@ -331,23 +429,29 @@ static int Service_Install(
         return LogWin32Error(GetLastError(), "cannot open service manager");
 
     // create service
-    serviceHandle = CreateService(managerHandle, PyString_AS_STRING(fullName),
-            PyString_AS_STRING(displayName), SERVICE_ALL_ACCESS,
-            SERVICE_WIN32_OWN_PROCESS, SERVICE_DEMAND_START,
-            SERVICE_ERROR_NORMAL, PyString_AS_STRING(command), NULL, NULL,
+    serviceHandle = CreateServiceW(managerHandle,
+            PyUnicode_AS_UNICODE(fullName), PyUnicode_AS_UNICODE(displayName),
+            SERVICE_ALL_ACCESS, SERVICE_WIN32_OWN_PROCESS, info.startType,
+            SERVICE_ERROR_NORMAL, PyUnicode_AS_UNICODE(command), NULL, NULL,
             NULL, NULL, NULL);
     if (!serviceHandle)
         return LogWin32Error(GetLastError(), "cannot create service");
 
     // set the description of the service, if one was specified
     if (info.description) {
-        sd.lpDescription = PyString_AS_STRING(info.description);
-        if (!ChangeServiceConfig2(serviceHandle, SERVICE_CONFIG_DESCRIPTION,
+        sd.lpDescription = PyUnicode_AS_UNICODE(info.description);
+        if (!ChangeServiceConfig2W(serviceHandle, SERVICE_CONFIG_DESCRIPTION,
                     &sd))
             return LogWin32Error(GetLastError(),
                     "cannot set service description");
     }
 
+    // if the service is one that should be automatically started, start it
+    if (info.startType == SERVICE_AUTO_START) {
+        if (!StartService(serviceHandle, 0, NULL))
+            return LogWin32Error(GetLastError(), "cannot start service");
+    }
+
     // close the service handles
     CloseServiceHandle(serviceHandle);
     CloseServiceHandle(managerHandle);
@@ -361,22 +465,26 @@ static int Service_Install(
 //   Uninstall the service with the given name.
 //-----------------------------------------------------------------------------
 static int Service_Uninstall(
-    char *programName,                  // name of program being run
-    char *name)                         // name of service
+    wchar_t *name)                      // name of service
 {
+    PyObject *fullName, *formatArgs, *nameObj;
     SC_HANDLE managerHandle, serviceHandle;
-    PyObject *fullName, *formatArgs;
+    SERVICE_STATUS statusInfo;
     udt_ServiceInfo info;
 
     // set up Python
-    if (Service_SetupPython(programName, &info) < 0)
+    if (Service_SetupPython(&info) < 0)
         return -1;
 
     // determine name of the service
-    formatArgs = Py_BuildValue("(s)", name);
+    nameObj = PyUnicode_FromWideChar(name, -1);
+    if (!nameObj)
+        return LogPythonException("cannot create service name obj");
+    formatArgs = PyTuple_Pack(1, nameObj);
     if (!formatArgs)
         return LogPythonException("cannot create service name tuple");
-    fullName = PyString_Format(info.nameFormat, formatArgs);
+    Py_CLEAR(nameObj);
+    fullName = PyUnicode_Format(info.nameFormat, formatArgs);
     if (!fullName)
         return LogPythonException("cannot create service name");
 
@@ -386,10 +494,11 @@ static int Service_Uninstall(
         return LogWin32Error(GetLastError(), "cannot open service manager");
 
     // create service
-    serviceHandle = OpenService(managerHandle, PyString_AS_STRING(fullName),
+    serviceHandle = OpenServiceW(managerHandle, PyUnicode_AS_UNICODE(fullName),
             SERVICE_ALL_ACCESS);
     if (!serviceHandle)
         return LogWin32Error(GetLastError(), "cannot open service");
+    ControlService(serviceHandle, SERVICE_CONTROL_STOP, &statusInfo);
     if (!DeleteService(serviceHandle))
         return LogWin32Error(GetLastError(), "cannot delete service");
     CloseServiceHandle(serviceHandle);
@@ -406,7 +515,7 @@ static int Service_Uninstall(
 static int Service_Run(
     udt_ServiceInfo *info)              // information about the service
 {
-    PyObject *temp;
+    PyObject *temp, *iniFileNameObj;
 
     // create an instance of the class which implements the service
     gInstance = PyObject_CallFunctionObjArgs(info->cls, NULL);
@@ -414,16 +523,19 @@ static int Service_Run(
         return LogPythonException("cannot create instance of service class");
 
     // initialize the instance implementing the service
-    LogMessageV(LOG_LEVEL_DEBUG, "initializing with config file %s",
-            gIniFileName);
-    temp = PyObject_CallMethod(gInstance, "Initialize", "s", gIniFileName);
+    LogMessageV(LOG_LEVEL_DEBUG, CX_SERVICE_INIT_MESSAGE, gIniFileName);
+    iniFileNameObj = PyUnicode_FromWideChar(gIniFileName, -1);
+    if (!iniFileNameObj)
+        return LogPythonException("failed to create ini file as string");
+    temp = PyObject_CallMethod(gInstance, "Initialize", "O", iniFileNameObj);
     if (!temp)
         return LogPythonException("failed to initialize instance properly");
-    Py_DECREF(temp);
+    Py_CLEAR(iniFileNameObj);
+    Py_CLEAR(temp);
 
     // run the service
     LogMessage(LOG_LEVEL_INFO, "starting up service");
-    if (Service_SetStatus(SERVICE_RUNNING) < 0)
+    if (Service_SetStatus(info, SERVICE_RUNNING) < 0)
         return LogWin32Error(GetLastError(), "cannot set service as started");
     temp = PyObject_CallMethod(gInstance, "Run", NULL);
     if (!temp)
@@ -445,17 +557,17 @@ static int Service_Run(
 //   Main routine for the service.
 //-----------------------------------------------------------------------------
 static void WINAPI Service_Main(
-    int argc,				// number of arguments
-    char **argv)			// argument values
+    int argc,				            // number of arguments
+    char **argv)			            // argument values
 {
     udt_ServiceInfo info;
 
-    if (Service_SetupPython(argv[0], &info) < 0)
+    if (Service_SetupPython(&info) < 0)
         return;
 
     // register the control function
     LogMessage(LOG_LEVEL_DEBUG, "registering control function");
-    gServiceHandle = RegisterServiceCtrlHandler("", Service_Control);
+    gServiceHandle = RegisterServiceCtrlHandlerEx("", Service_Control, &info);
     if (!gServiceHandle) {
         LogWin32Error(GetLastError(),
                 "cannot register service control handler");
@@ -464,7 +576,7 @@ static void WINAPI Service_Main(
 
     // run the service
     if (Service_Run(&info) < 0) {
-        Service_SetStatus(SERVICE_STOPPED);
+        Service_SetStatus(&info, SERVICE_STOPPED);
         return;
     }
 
@@ -479,7 +591,7 @@ static void WINAPI Service_Main(
     // otherwise, the service terminated normally by some other means
     } else {
         LogMessage(LOG_LEVEL_INFO, "stopping service (internally)");
-        Service_SetStatus(SERVICE_STOPPED);
+        Service_SetStatus(&info, SERVICE_STOPPED);
     }
 }
 
@@ -488,39 +600,44 @@ static void WINAPI Service_Main(
 // main()
 //   Main routine for the service.
 //-----------------------------------------------------------------------------
-int main(int argc, char **argv)
+int wmain(int argc, wchar_t **argv)
 {
-    char *configFileName = NULL;
+    wchar_t *configFileName = NULL;
 
     SERVICE_TABLE_ENTRY table[] = {
         { "", (LPSERVICE_MAIN_FUNCTION) Service_Main },
         { NULL, NULL }
     };
 
-    gIniFileName[0] = '\0';
+    // initialize Python
+    if (InitializePython(argc, argv) < 0)
+        return 1;
+
+    // check for arguments and perform install/uninstall as requested
+    gIniFileName[0] = L'\0';
     if (argc > 1) {
-        if (stricmp(argv[1], "--install") == 0) {
+        if (wcsicmp(argv[1], CX_SERVICE_INSTALL_OPTION) == 0) {
             if (argc == 2) {
                 fprintf(stderr, "Incorrect number of parameters.\n");
-                fprintf(stderr, "%s --install <NAME> [<CONFIGFILE>]", argv[0]);
+                fprintf(stderr, CX_SERVICE_INSTALL_USAGE, argv[0]);
                 return 1;
             }
             if (argc > 3)
                 configFileName = argv[3];
-            if (Service_Install(argv[0], argv[2], configFileName) < 0) {
+            if (Service_Install(argv[2], configFileName) < 0) {
                 fprintf(stderr, "Service not installed. ");
                 fprintf(stderr, "See log file for details.");
                 return 1;
             }
             fprintf(stderr, "Service installed.");
             return 0;
-        } else if (stricmp(argv[1], "--uninstall") == 0) {
+        } else if (wcsicmp(argv[1], CX_SERVICE_UNINSTALL_OPTION) == 0) {
             if (argc == 2) {
                 fprintf(stderr, "Incorrect number of parameters.\n");
-                fprintf(stderr, "%s --uninstall <NAME>", argv[0]);
+                fprintf(stderr, CX_SERVICE_UNINSTALL_USAGE, argv[0]);
                 return 1;
             }
-            if (Service_Uninstall(argv[0], argv[2]) < 0) {
+            if (Service_Uninstall(argv[2]) < 0) {
                 fprintf(stderr, "Service not installed. ");
                 fprintf(stderr, "See log file for details.");
                 return 1;
@@ -528,10 +645,10 @@ int main(int argc, char **argv)
             fprintf(stderr, "Service uninstalled.");
             return 0;
         }
-        strcpy(gIniFileName, argv[1]);
+        wcscpy(gIniFileName, argv[1]);
     }
 
-
+    // run the service normally
     return StartServiceCtrlDispatcher(table);
 }
 
